==> moagg_0.11.orig.tar.gz: moagg-0.11/libMoaggObjects/include/Grenade.h <==
#ifndef GRENADE_H
#define GRENADE_H

#include "MovingObjectBase.h"

class MortarBarrel;
class MortarBase;

#define GRENADE_VELOCITY 300


//----------------------------------------------------------------------------
class GrenadeSurfaces : public SingleSurfacesBase
{
    SURFACES_SINGLETON_OBJECT(GrenadeSurfaces);
};


//----------------------------------------------------------------------------
#undef DECLARE_WARHEAD_STRATEGY
#define DECLARE_WARHEAD_STRATEGY(c) \
    class c; \
    friend class c; \
    class c : public WarheadStrategy \
    { \
        STRATEGY_OBJECT(c); \
      public: \
        c() {} \
        ~c() {} \
        void onCreateExplosionParticles(GrenadeBase *grenade); \
    }


//----------------------------------------------------------------------------
class GrenadeBase : public MovingObjectBase
{
  public:
    //------------------------------------------------------------------------
    enum EnumWarheadStrategy
    {
        WH_NONE,
        WH_STARBURST
    };

  private:

    //------------------------------------------------------------------------
    /**
     * A base class for all warhead strategies.
     */
    class WarheadStrategy
    {
      public:
        virtual ~WarheadStrategy() {}

        virtual void onCreateExplosionParticles(GrenadeBase *grenade) = 0;

        static WarheadStrategy *getStrategy(EnumWarheadStrategy s);

      protected:
        WarheadStrategy() {}
    };

    DECLARE_WARHEAD_STRATEGY(NoWarheadStrategy);
    DECLARE_WARHEAD_STRATEGY(StarBurstWarheadStrategy);


  public:

    //------------------------------------------------------------------------
    GrenadeBase(EnumWarheadStrategy warhead, const ObjectBase *creator);
    virtual ~GrenadeBase();

    //------------------------------------------------------------------------
    inline bool isExplode() const { return m_explode; }

    /**
     * Creates some explosion particles on the grenade's position.
     */
    void createExplosionParticles();

    //------------------------------------------------------------------------
    void update();

  protected:
    //------------------------------------------------------------------------
    inline void setLifeCount(unsigned lifeCount)
    {
        m_lifeCount = lifeCount;
    }

  private:
    //------------------------------------------------------------------------
    DECLARE_OBJECT_VISITOR_API();

    //------------------------------------------------------------------------
    /// The number of frames, this grenade will live.
    unsigned m_lifeCount;

    /// Set to true, if this grenade shall be removed from the playground.
    bool m_explode;

    //------------------------------------------------------------------------
    WarheadStrategy *m_warheadStrategy;
};

//----------------------------------------------------------------------------
class MortarGrenade : public GrenadeBase
{
  public:
    MortarGrenade(MortarBase *creator, const MortarBarrel *barrel);
    ~MortarGrenade();
};

#endif //GRENADE_H

==> moagg_0.16.orig.tar.gz: moagg-0.16/libMoaggObjects/include/Grenade.h <==
#ifndef GRENADE_H
#define GRENADE_H

#include "MovingObjectBase.h"

class MortarBarrel;
class MortarBase;

#define GRENADE_VELOCITY 300


//----------------------------------------------------------------------------
class GrenadeSurfaces : public SingleSurfacesBase
{
    SURFACES_SINGLETON_OBJECT(GrenadeSurfaces);
};


//----------------------------------------------------------------------------
#undef DECLARE_WARHEAD_STRATEGY
#define DECLARE_WARHEAD_STRATEGY(c) \
    class c; \
    friend class c; \
    class c : public WarheadStrategy \
    { \
        STRATEGY_OBJECT(c); \
      public: \
        c() {} \
        ~c() {} \
        void onCreateExplosionParticles(const GrenadeBase *grenade) const; \
    }


//----------------------------------------------------------------------------
class GrenadeBase : public MovingObjectBase
{
  public:
    //------------------------------------------------------------------------
    enum EnumWarheadStrategy
    {
        WH_NONE,
        WH_STARBURST
    };

  private:

    //------------------------------------------------------------------------
    /**
     * A base class for all warhead strategies.
     */
    class WarheadStrategy
    {
      public:
        virtual ~WarheadStrategy() {}

        virtual void onCreateExplosionParticles(const GrenadeBase *grenade) const = 0;

        static WarheadStrategy *getStrategy(EnumWarheadStrategy s);

      protected:
        WarheadStrategy() {}
    };

    DECLARE_WARHEAD_STRATEGY(NoWarheadStrategy);
    DECLARE_WARHEAD_STRATEGY(StarBurstWarheadStrategy);


  public:

    //------------------------------------------------------------------------
    GrenadeBase(EnumWarheadStrategy warhead, const unsigned creatorId);
    virtual ~GrenadeBase();

    //------------------------------------------------------------------------
    inline bool isExplode() const { return m_explode; }

    //------------------------------------------------------------------------
    void update();

    //------------------------------------------------------------------------
    /**
     * Called via the CreateExplosionParticleConstVisitor.
     */
    inline void onCreateExplosionParticles() const
    {
        m_warheadStrategy->onCreateExplosionParticles(this);
    }

  protected:
    //------------------------------------------------------------------------
    inline void setLifeCount(unsigned lifeCount)
    {
        m_lifeCount = lifeCount;
    }

  private:
    //------------------------------------------------------------------------
    DECLARE_OBJECT_VISITOR_API();

    //------------------------------------------------------------------------
    /// The number of frames, this grenade will live.
    unsigned m_lifeCount;

    /// Set to true, if this grenade shall be removed from the playground.
    bool m_explode;

    //------------------------------------------------------------------------
    WarheadStrategy *m_warheadStrategy;
};

//----------------------------------------------------------------------------
class MortarGrenade : public GrenadeBase
{
  public:
    MortarGrenade(MortarBase *creator, const MortarBarrel *barrel);
    ~MortarGrenade();
};

#endif //GRENADE_H
